@@ -1,5 +1,5 @@
-#ifndef NP_UTILS_H
-#define NP_UTILS_H
+#ifndef NP_UTILS_H_INCLUDED
+#define NP_UTILS_H_INCLUDED
 /* Header file for nagios plugins utils.c */
 
 /* This file should be included in all plugins */
@@ -178,9 +178,10 @@ void strntolower (char * test_char, int size);
     Critical range\n")
 
 #define UT_CONN_TIMEOUT _("\
- -t, --timeout=INTEGER\n\
-    Seconds before connection times out (default: %d)\n")
-
+ -t, --timeout=INTEGER:<timeout state>\n\
+    Seconds before connection times out (default: %d)\n\
+    Optional \":<timeout state>\" can be a state integer (0,1,2,3) or a state STRING\n")
+ 
 #define UT_PLUG_TIMEOUT _("\
  -t, --timeout=INTEGER\n\
     Seconds before plugin times out (default: %d)\n")
@@ -210,4 +211,4 @@ The nagios plugins come with ABSOLUTELY NO WARRANTY. You may redistribute\n\
 copies of the plugins under the terms of the GNU General Public License.\n\
 For more information about these matters, see the file named COPYING.\n")
 
-#endif /* NP_UTILS_H */
+#endif /* NP_UTILS_H_INCLUDED */
